/*
 
 DESCRIPTION:
 Include file for TLOG.
 
 Copyright 2019 Brian Cornell.
 
 Permission is hereby granted, free of charge, to any person obtaining a copy of
 this software and associated documentation files (the "Software"), to deal in
 the Software without restriction, including without limitation the rights to
 use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 of the Software, and to permit persons to whom the Software is furnished to do
 so, subject to the following conditions:
 
 The above copyright notice and this permission notice shall be included in all
 copies or substantial portions of the Software.
 
 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 DEPENDENCIES:  none
 
 VERSION:  1.0
 
 MODIFICATION HISTORY:
 
 19 Feb 19, BWC, first release.
 
 
 */

#ifndef tlog_h
#define tlog_h
#endif

#define VERMAJOR        1
#define VERMINOR        0
#define VERBUILD        29

#define TRUE            1
#define FALSE           0

#define BASE_TEN        10              //For string conversions

#define WAIT_LOG_STOP   2               //Number of seconds to wait for logging to stop
#define WAIT_LOG_LOOP   1               //Number of seconds between process loop iterations in log mode

#define SYS_EPOCH_YEAR  1900            //Epoch for C library
#define LOG_EPOCH_YEAR  2000            //Epoch for logger HW

#define FMODE_LOG_DEV   "r+"            //Logging device open mode
#define FMODE_CONF      "r"             //Config file open mode
#define FMODE_LOG_FIL   "w"             //Open & truncate, write only

#define PROC_ID         0x3085          //Processor ID of logger HW
#define SIZ_PROC_ID     4               //Size of PROC_ID when printed to string w/o terminating NULL

//Command line argument positions
#define POS_ARG_PRG     0
#define POS_ARG_DEV     1

//Ansi terminal control & characters
#define ANSIESC         "\e["
#define ANSICLRSCR      "2J"
#define ANSICPOS        'H'
#define ANSIELINE       'K'
#define ANSISAVE        "\e7"
#define ANSIRESTORE     "\e8"
#define ACP_SZ_STR      4
#define ACP_MAX_ROW     25
#define ACP_MAX_COL     80
#define CHR_CR          0x0D
#define CHR_LF          0x0A
#define CHR_NULL        0x00
#define CHR_COLON       ':'
#define CHR_SEMICOLON   ';'
#define CHR_SPACE       ' '

//Cursor positions
#define POS_HDR_R       1
#define POS_HDR_C       6
#define POS_CMD_R       16
#define POS_CMD_C       1
#define POS_HDR_LBL_R   3
#define POS_HDR_LBL_C   10
#define POS_AMB_LBL_R   4
#define POS_AMB_LBL_C   2
#define POS_AMB_VAL_R   4
#define POS_AMB_VAL_C   10
#define POS_CH2_LBL_R   6
#define POS_CH2_LBL_C   2
#define POS_CH2_VAL_R   6
#define POS_CH2_VAL_C   10
#define POS_CH4_LBL_R   8
#define POS_CH4_LBL_C   2
#define POS_CH4_VAL_R   8
#define POS_CH4_VAL_C   10
#define POS_CH5_LBL_R   10
#define POS_CH5_LBL_C   2
#define POS_CH5_VAL_R   10
#define POS_CH5_VAL_C   10
#define POS_SEQ_LBL_R   12
#define POS_SEQ_LBL_C   5
#define POS_SEQ_VAL_R   12
#define POS_SEQ_VAL_C   10

//Config file field positions
#define POS_CFG_LBL     1               //Display name for channel (max 8 char)
#define POS_CFG_NTC     2               //Sensor type indicator
#define POS_CFG_MVC     3               //mV/C for active sensor
#define POS_CFG_ZOF     4               //Active sensor offset for 0C
#define POS_CFG_CE1     5               //NTC coefficient
#define POS_CFG_CE2     6               //NTC coefficient
#define POS_CFG_CE3     7               //NTC coefficient
#define POS_CFG_CE4     8               //NTC coefficient

//Analog channel mapping
#define MAP_TMP_AMB     1
#define MAP_TMP_CH2     2
#define MAP_TMP_CH4     4
#define MAP_TMP_CH5     5
#define MAP_TMP_MIN     MAP_TMP_AMB     //Minimum valid channel #
#define MAP_TMP_MAX     MAP_TMP_CH5     //Maximum valid channel #
#define SUM_TMP_CHL     (MAP_TMP_AMB + MAP_TMP_CH2 + MAP_TMP_CH4 + MAP_TMP_CH5)

//Temperature calculation
#define CALC_TMP_COEF   10              //mV per degrees C
#define CALC_TMP_ZOFF   50              //Zero degrees C offset

//Temperature limits
#define LIM_TMP_MIN     -50
#define LIM_TMP_MAX     150

//Current sense
#define ADC_OFFSET      512
#define ADC_MVAMP       0.185

//ADC coefficients
#define ADC_MAX_VAL     1023

#define SIZ_ARY_TMP     6               //Size of array for temp objects
#define SIZ_RSP_CMD     52              //Largest # of chars to process in logger handshake
#define SIZ_RSP_STRT    3               //# char to read in response to 'start logging' command
#define SIZ_ARGC_MIN    2               //Minimum # arguments
#define SIZ_ARGV_CMD    2               //Position for command
#define SIZ_ARGC_BASE   4               //Arguments required for 'log' command
#define SIZ_ARGV_BASE   (SIZ_ARGC_BASE - 1) //Base argument for 'log' command
#define SIZ_MAX_BASE    202             //Maximum size of user-supplied base path
#define SIZ_LOG_NAME    16              //Space required to add date & extension
#define SIZ_LOG_PATH    (SIZ_MAX_BASE + SIZ_LOG_NAME)
#define SIZ_BUF_IDS     (SIZ_PROC_ID + 2)   //Size of buffer to hold processor ID string
#define SIZ_REC_HDR     1               //# header records to read
#define SIZ_BUF_LBL     10              //Size of buffer label string
#define SIZ_BUF_REC     256             //Buffer for file record
#define SIZ_BUF_VAR     64              //Size to hold config file variables

//Program strings
#define STR_PRGNAME     "tlog"         //Program name
#define STR_EXT_CFG     ".conf"         //Config extension

//Print & display formats
#define FMT_LBL_AMB     "%s"
#define FMT_LBL_CH2     "%s"
#define FMT_LBL_CH4     "%s"
#define FMT_LBL_CH5     "%s"
#define FMT_LBL_SEQ     "Seq #"
#define FMT_LBL_VDD     "Vdd "
#define FMT_DAT_PATH    "%s-%c-%4.4d%2.2d%2.2d.%s"
#define FMT_LOG_HDSK    "st\r\rso b\r\rid\r\r"  //Start logging handshake
#define FMT_LOG_EPOCH   "sd %02d%02d%02d%02d%02d%02d\r\r"
#define FMT_LOG_CONF    "sa 1 p 2\r\rsa 2 p 2\r\rsa 4 p 2\r\rsa 5 p 2\r\r"
#define FMT_LOG_LVS     "sv 3.3\r\r"
#define FMT_LOG_START   "sl\r\r"
#define FMT_LOG_STOP    "st\r\r"                //Stop logging command
#define FMT_LOG_GID     "id\r\r"                //Get ID command
#define FMT_BUF_IDS     "%4.4X"                 //Format processor ID from Hex
#define FMT_ERR_SEQ     "\nSequence # mismatch:  expected %u, got %u."
#define FMT_ERR_STDERR  "\n%s:  %s, errno=%d."
#define FMT_TRM_SIGNAL  "\n%s terminating on signal %d.\n"
#define FMT_PRG_DESC    "Logger temperature display, v%d.%d build %d.  (c) 2019 Brian Cornell."
#define FMT_PRG_SYNTX   "\nSyntax:  %s <device> (flags).  Optional flags:"
#define FMT_PRG_EXIT    "\n\n"
#define FMT_PRG_CMDL    "\n\t%s - %s"
#define FMT_HDR_FIELDS  "%s,%s"
#define FMT_ERR_RTDMP   "\ndata:  %2X,%u.%u.%u,%u,%u,%u,%2X,%2X,%2X"
#define FMT_ERR_RECT    "\nData: 0x"
#define FMT_ERR_RECTV   " %.2X"
#define FMT_TMP_DSP     "%.2f\t%.2f\t%.2f\t%.2f"
#define FMT_SEQ_DSP     "%u\t%s%.2f"
#define FMT_MSG_EXIT    "Type Ctrl-C to exit."
#define FMT_HDR_LABEL   "Now\tMin\tMax\tDelta"
#define FMT_OUT_LOG     "\n%u,%2.2d/%2.2d/%d,%2.2d:%2.2d:%2.2d,%u,%s,%.2f,%.2f,%.2f,%.2f"
#define FMT_HDR_LOG     "Seq #,Date,Time,Channel,Label,Now,Min,Max,Dif"
#define FMT_CFG_DUMP    "\nChannel %d:\n\tlbl=%s\n\tntc=%d\n\tmvc=%d\n\tzof=%d\n\tce1=%f\n\tce2=%f\n\tce3=%f\n\tce4=%f"
#define FMT_MSG_LOG     "\nWriting data to %s"

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Manage collection & display of temperatures
 +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
typedef struct {
    int   ch;                   //Channel
    int   ntc;                  //Non-zero if NTC
    int   mvc;                  //Active sensor mV/C
    int   zof;                  //Active sensor offset for 0C, degrees C
    int   row;                  //Display row
    int   col;                  //Display column
    float ce1;                  //NTC 3rd order transfer fct coefficient 1
    float ce2;                  //NTC 3rd order transfer fct coefficient 2
    float ce3;                  //NTC 3rd order transfer fct coefficient 3
    float ce4;                  //NTC 3rd order transfer fct coefficient 4
    float now;                  //Current temp
    float min;                  //Minimum temp
    float max;                  //Maximum temp
    float dif;                  //Max-min
    char  lbl[SIZ_BUF_LBL];     //Buffer for display label
} temp;

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Command line flags
 +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
typedef struct {
    const char *str;
    const char *desc;
    char       *value;
} cmdf;

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Command line processing
 +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
//Flag bit mapping
#define FLG_NONE        0x0000
#define FLG_BIT_SET     0x0001
#define FLG_DIFF_CONF   0x0001
#define FLG_DUMP_CONF   0x0002
#define FLG_LOG_FILE    0x0004

//Flag array position
#define FLG_A_DIFF_CONF 0
#define FLG_A_DUMP_CONF 1
#define FLG_A_LOG_FILE  2

cmdf Flags[] = {
    {"-c","use different config <file>"},
    {"-d","dump config"},
    {"-l","log data to <file>"}
};

#define SIZ_A_FLG       sizeof(Flags) / (sizeof(const char *) * 3)


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Return codes & messages
 +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
#define RC_SUCCESS      0
#define RC_NOARG        1
#define RC_BADARG       2
#define RC_DEVOFAIL     3
#define RC_DEVDFAIL     4
#define RC_LOGHFAIL     5
#define RC_LOGAFAIL     6
#define RC_INVCHANL     7
#define RC_LOGFAIL      8
#define RC_BADCFG       9
#define RC_SIGEXIT      10
#define RC_SIGERROR     11
#define RC_RECTYPE      12
#define RC_SEQNUMBR     13

const char *retCode[] = {
    "success",
    "no arguments",
    "bad or missing arguments",
    "couldn't open log device",
    "failed to start logging on device",
    "couldn't read / validate header",
    "couldn't read logger data",
    "invalid channel",
    "data log file error",
    "problem with config file",
    "CTRL-C or TERM signal received",
    "error installing signal handler",
    "invalid record type",
    "bad sequence #"
};


int main(int, char *[]);
void getTemp(int, int, temp *);
float getVolts(int, int, int);
float getAmps(int, int, int, float);
unsigned int TimeSinceEpoch(logtm *);
int startLog(FILE *);
int stopLog(FILE *);
void sigHandler(int);
time_t readHeader(FILE *);
void ansiClrScr(void);
void ansiEraseLine(void);
void ansiCursorPos(unsigned char, unsigned char);
void ansiCursorSav(void);
void ansiCursorRst(void);
const char *getCfgVar(char *, int);
int strnl(unsigned char *, int);
int loadCfg(char *, int, temp *, int);


//End file.
